(require 'gtkrc)

;; This file is obviously heavily based on clean-gtk.
;; To be honest, the gtk colouring code is still here because I
;; couldn't find out how to remove it without damaging the theme...
;; 
;; Modifications done by Alexander Graefe <gatekeeper@hannigan.com>

(let*
    ((make-image
      (lambda (file)
	(copy-image (make-image file))))

     (recolor-image
      (lambda (img color bright)
	(when (and (imagep img) (colorp color))
	  (let*
	      ((rgb (color-rgb color))
	       (red (nth 0 rgb))
	       (green (nth 1 rgb))
	       (blue (nth 2 rgb)))
	    (if bright
		(setq bright 255)
	      (setq bright 224))
	    ;; use the color modifiers
	    (set-image-modifier
	     img nil '(255 255 255))
	    (set-image-modifier
	     img 'red (list (/ red 256) bright (/ red 256)))
	    (set-image-modifier
	     img 'green (list (/ green 256) bright (/ green 256)))
	    (set-image-modifier
	     img 'blue (list (/ blue 256) bright (/ blue 256)))))))

     (normal-bl-i (list (make-image "bl.png") (make-image "bl.png")))
     (normal-br-i (list (make-image "br.png") (make-image "br.png")))
     (normal-tl-i (list (make-image "tl.png") (make-image "tl.png")))
     (normal-tr-i (list (make-image "tr.png") (make-image "tr.png")))

     (normal-left-i
      (list (set-image-border (make-image "left.png") 0 0 1 2)
	    (set-image-border (make-image "left.png") 0 0 1 2)))
     (normal-right-i
      (list (set-image-border (make-image "right.png") 0 0 1 2)
	    (set-image-border (make-image "right.png") 0 0 1 2)))
     (normal-top-i
      (list (set-image-border (make-image "top.png") 1 2 0 0)
	    (set-image-border (make-image "top.png") 1 2 0 0)))
     (normal-bottom-i
      (list (set-image-border (make-image "bottom.png") 1 2 0 0)
	    (set-image-border (make-image "bottom.png") 1 2 0 0)))

     (normal-menu-i
      (list (make-image "menu.png") (make-image "menu.png") nil
	    (make-image "menu-c.png")))
     (normal-minimize-i
      (list (make-image "minimize.png") (make-image "minimize.png") nil
	    (make-image "minimize-c.png")))
     (normal-maximize-i
      (list (make-image "maximize.png") (make-image "maximize.png") nil
	    (make-image "maximize-c.png")))
     (normal-close-i
      (list (make-image "close.png") (make-image "close.png") nil
	    (make-image "close-c.png")))

     (normal-title-i
      (list (set-image-border (make-image "title.png") 7 7 2 3)
	    (set-image-border (make-image "title.png") 7 7 2 3)))

     (normal-images
      (list normal-bl-i normal-br-i normal-tl-i normal-tr-i
	    normal-left-i normal-right-i normal-top-i normal-bottom-i
	    normal-menu-i normal-minimize-i normal-maximize-i normal-close-i
	    normal-title-i))

     (rebuild-frames
      (lambda ()
	(when gtkrc-background
	  (mapc (lambda (images)
		  (recolor-image (nth 0 images) (car gtkrc-background) nil)
		  (recolor-image (nth 1 images) (car gtkrc-background) t)
		  (recolor-image (nth 3 images) (car gtkrc-background) t))
		normal-images))
	(mapc (lambda (w)
		(when (eq (window-get w 'current-frame-style) 'Spartan)
		  (set-window-frame-style w 'Spartan)))
	      (managed-windows))))

     (normal-frame `(((background . ,normal-tl-i)
		      (left-edge . -13)
		      (top-edge . -33)
		      (class . top-left-corner))
		     ((background . ,normal-bl-i)
		      (left-edge . -13)
		      (bottom-edge . -12)
		      (class . bottom-left-corner))
		     ((background . ,normal-br-i)
		      (right-edge . -12)
		      (bottom-edge . -12)
		      (class . bottom-right-corner))
		     ((background . ,normal-tr-i)
		      (right-edge . -12)
		      (top-edge . -33)
		      (class . top-right-corner))

		     ((background . ,normal-menu-i)
		      (left-edge  . 11)
		      (top-edge . -24)
		      (class . menu-button))

;		     ((background . ,normal-top-i)
;                      (left-edge . 9)
;                      (right-edge . 8)
;                      (top-edge . -22)
;                      (class . top-border))
		     ((background . ,normal-left-i)
                      (left-edge . -5)
                      (top-edge . 0)
                      (bottom-edge . 8)
                      (class . left-border))
		     ((background . ,normal-bottom-i)
                      (left-edge . 10)
                      (right-edge . 10)
                      (bottom-edge . -3)
                      (class . bottom-border))
		     ((background . ,normal-right-i)
                      (right-edge . -4)
                      (bottom-edge . 8)
                      (top-edge . 0)
                      (class . right-border))
		     ((background . ,normal-maximize-i)
                      (right-edge . 25)
                      (top-edge . -24)
                      (class . maximize-button))
                     ((background . ,normal-minimize-i)
                      (right-edge . 37)
                      (top-edge . -24)
                      (class . iconify-button))
		     ((background . ,normal-close-i)
		      (right-edge  . 0)
		      (top-edge . -24)
		      (class . close-button))
		     ((background . ,normal-title-i)
		      (left-edge . 35)
		      (right-edge . 49)
		      (top-edge . -24)
		      (class . title)
		      (foreground . ("grey","white"))
		      (text . ,window-name)
		      (x-justify . left)
		      (y-justify . center))))

     (shaped-frame `(((background . ,normal-tl-i)
		      (left-edge . -3)
		      (top-edge . -22)
		      (class . top-left-corner))
		     ((background . ,normal-bl-i)
		      (left-edge . -3)
		      (top-edge . -7)
		      (class . bottom-left-corner))
		     ((background . ,normal-br-i)
		      (right-edge . -3)
		      (top-edge . -7)
		      (class . bottom-right-corner))
		     ((background . ,normal-tr-i)
		      (right-edge . -3)
		      (top-edge . -22)
		      (class . top-right-corner))
		     ((background . ,normal-menu-i)
		      (left-edge  . 0)
		      (top-edge . -20)
		      (class . menu-button))
		     ((background . ,normal-close-i)
		      (right-edge . 0)
		      (top-edge . -20)
		      (class . close-button))
		     ((background . ,normal-maximize-i)
		      (right-edge . 16)
		      (top-edge . -20)
		      (class . maximize-button))
		     ((background . ,normal-minimize-i)
		      (right-edge . 32)
		      (top-edge . -20)
		      (class . iconify-button))
		     ((background . ,normal-title-i)
		      (left-edge . 17)
		      (right-edge . 48)
		      (top-edge . -20)
		      (class . title)
		      (foreground . ("grey","white"))
		      (text . ,window-name)
		      (x-justify . 8)
		      (y-justify . center))))

     (transient-frame `(((background . ,normal-tl-i)
			 (left-edge . -12)
			 (top-edge . -33)
			 (class . top-left-corner))
			((background . ,normal-left-i)
			 (left-edge . -4)
			 (top-edge . 0)
			 (bottom-edge . 0)
			 (class . left-border))
			((background . ,normal-bl-i)
			 (left-edge . -13)
			 (bottom-edge . -12)
			 (class . bottom-left-corner))
			((background . ,normal-bottom-i)
			 (left-edge . 11)
			 (right-edge . 11)
			 (bottom-edge . -3)
			 (class . bottom-border))
			((background . ,normal-br-i)
			 (right-edge . -13)
			 (bottom-edge . -12)
			 (class . bottom-right-corner))
			((background . ,normal-right-i)
			 (right-edge . -4)
			 (bottom-edge . 0)
			 (top-edge . 0)
			 (class . right-border))
			((background . ,normal-tr-i)
			 (right-edge . -12)
			 (top-edge . -33)
			 (class . top-right-corner))
			((background . ,normal-title-i)
			 (left-edge . 12)
			 (right-edge . 12)
			 (top-edge . -24)
			 (class . title))))

     (shaped-transient-frame `(((background . ,normal-title-i)
				(left-edge . 0)
				(right-edge . 0)
				(top-edge . -24)
				(class . title)))))

  (add-frame-style 'Spartan
		   (lambda (w type)
		     (cond ((eq type 'shaped)
			    shaped-frame)
			   ((eq type 'transient)
			    transient-frame)
			   ((eq type 'shaped-transient)
			    shaped-transient-frame)
			   ((eq type 'unframed)
			    nil-frame)
			   (t
			    normal-frame))))

  (unless batch-mode
    (rebuild-frames))

  (gtkrc-call-after-changed rebuild-frames))


